#!/bin/env python

from argparse import ArgumentParser
from logging import info, debug, error, basicConfig, INFO, DEBUG
from pathlib import Path
from tomllib import load
from requests import Response, utils, get, post
from sys import exit

# import bs4


def build_arguent_parser() -> ArgumentParser:
    """A function to build the argument parser.

    Returns:
        ArgumentParser: The initialized argument parser object.
    """

    parser = ArgumentParser(
        prog="mylarp-api",
        description="A python API for mylarp",
        epilog="",
    )
    parser.add_argument(
        "-c",
        "--config",
        default="config.toml",
        help='(Optional) Path to a custom config.toml file. Default: "config.toml"',
    )
    parser.add_argument(
        "-u",
        "--uri",
        default="/",
        help="The uri of the mylarp page to parse.",
    )
    parser.add_argument(
        "-v",
        "--verbose",
        help="(Optional) Output debug information.",
        action="store_true",
    )
    return parser


def parse_config(config_file: str) -> dict:
    """A parser function for the config.toml

    Args:
        config_file (str): The path to the config file.

    Returns:
        dict: The resulting dictionary object.
    """

    config_file_path = Path(config_file).absolute()

    if not config_file_path.exists():
        error(f"Specified config file path does not exist: {config_file_path}")

    with open(config_file, "rb") as conf:
        config = load(conf)

    debug(config)
    return config


def parse_mylarp_page(root: str, uri: str, cookies: dict) -> Response:
    """A function to attempt to parse a page in MyLARP.

    Args:
        root (str): Specified root url for mylarp.
        uri (str): Specified uri for mylarp page.
        cookies (dict): Login cookies from previous attempted login.

    Returns:
        Response: The Response object from the attempted page request.
    """

    get_url = f"{root}{uri}"
    info(f"Attempting to parse from {get_url}")
    response = get(url=get_url, cookies=cookies)
    debug(response.text)
    if not response.ok:
        error(f"Invalid API request, server returned: {response.status_code}")
    return response


def attempt_login(paths: dict, auth: dict) -> Response:
    """Use the provided credentials to attempt a log in.

    Args:
        paths (dict): Parsed config uri paths.
        auth (dict): Parsed authentication config from config file.

    Returns:
        Response: The Response object from the attempted login request.
    """

    email = utils.quote(auth["email"])
    password = utils.quote(auth["password"])

    info("Attempting log in...")
    debug(
        f"""
Using credentials:
- Email: {email}
- Password: {password}
          """
    )
    post_url = f"{paths["root"]}{paths["login"]}"
    post_data = f"email={email}&pword={password}"
    response = post(
        url=post_url,
        data=post_data,
    )
    debug(response.text)
    return response


def main():
    parser = build_arguent_parser()
    args = parser.parse_args()
    if args.verbose:
        basicConfig(level=DEBUG)
    else:
        basicConfig(level=INFO)
    config = parse_config(args.config)
    login_response = attempt_login(config["paths"], config["auth"])
    login_cookies = login_response.cookies.get_dict()
    dashboard_response = parse_mylarp_page(
        config["paths"]["root"],
        config["paths"]["dashboard"],
        login_cookies,
    )
    debug(dashboard_response.text)
    exit(0)


if __name__ == "__main__":
    main()
